{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command module called by _CMD.CMD 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120306   011   separate _CMD.CMD module
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

' toggleLF setting lost on return


CON
  _clkmode = _hw#_clkmode
  _xinfreq = _hw#_xinfreq

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200



OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef"             'fixed hub definitions ($7FFF... down)

  hwdet : "DetectHardware_001"

  fat   : "SD-MMC_FATEngine_002.spin"
  taf   : "SD-MMC_FATEngine_002.spin"
  str   : "ASCII0_STREngine.spin"
  sio   : "pcFullDuplexSerial2FC" 

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

    long hardware               'hw def & pins
    long DO,CLK,DI,CS, InputPins, OutputPins

PUB shell | z,z0,z1,z2,z3,z4

' LineFeedOn := false                                   
'''''  long[_hub#_toggleLF] := false                         ' turn off auto lf after cr

'detect the hardware
  hardware := hwdet.DetectHardware
  if hardware == -1
    HardwareFail  
  DO  :=  hardware        & $1F
  CLK := (hardware >> 8)  & $1F
  DI  := (hardware >> 16) & $1F
  CS  := (hardware >> 24) & $1F
  case hardware >> 29
    1 :    InputPins  := _hw#_siRB1                     'RamBlade
           OutputPins := _hw#_soRB1
    2 :    InputPins  := _hw#_siPin                     'TriBlade
           OutputPins := _hw#_soPin
    3 :    InputPins  := _hw#_siPin                     'RamBlade3
           OutputPins := _hw#_soPin
    other: InputPins  := _hw#_siPin                     '??? use default P31/P30
           OutputPins := _hw#_soPin

'==================
'set parameters in hub for other modules
  long[_hub#_SDPins]  := hardware & $1F1F1F1F           
  long[_hub#_SIOPins] := _hw#_sioMode << 16 | OutputPins << 8 | InputPins
  long[_hub#_SIOBaud] := _hw#_sioBaud                              
  long[_hub#_toggleLF]:= false
' str.copy(string("dir *.bin"),_hub#_stringBuffer)           
'==================
  

  pauseForMilliseconds(500)
    
  z  :=  long[_hub#_sioPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
' z3 := (z >> 24) & $1F
  z4 :=  long[_hub#_sioBaud]
  sio.AddPort(0, z0, z1, -1, -1, 0, z2, z4)
' sio.AddPort(0, InputPins, OutputPins, -1, -1, 0, 0, _hw#_sioBaud)
  sio.start                                             ' start the com port

  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
' ifnot( fat.FATEngineStart(DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))
' ifnot( fat.FATEngineStart(DO, CLK, DI, CS, -1,     -1,     -1,       -1,       -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card
{
  PrintChar(_clearScreenCharacter)                      ' clear screen
  PrintDosVer                                           ' program message
  PrintStringCR(string(" Type Help for command listing"))
}
  ifnot(fat.partitionMounted)
    printstring(string("Card Not Mounted "))   

  sio.rxflush(0)                                        ' in case try to send an xmodem transfer
  crlf
  repeat

    result := shellLine(string("cmd->"))
    str.copy(result,@commandline)      ' save for later on if this is a command line with parameters eg "xmodem r filename.txt"
    crlf

'================
    str.copy(@commandline,_hub#_stringBuffer)           ' pass commandline to next program
'=================


    result := \shellCommands(result)
    PrintStringCR(result)

    if(fat.partitionError)
      crlf

PRI executeCommand(fname, stringPointer)
'=================
    \fat.bootPartition(fname)                           ' boot the module
    'only here on error
    PrintStringCR(string("=== error ==="))
    waitcnt(clkfreq/1000 * 500 + cnt)                   ' delay 500ms
    reboot
'=================


PRI shellCommands(stringPointer)

  stringPointer := str.tokenizeString(stringPointer)

  programClear(stringPointer)
  programEcho(stringPointer)

  programReboot(stringPointer)
  programBoot(stringPointer)
  programBin(stringPointer)

  programChangeDirectory(stringPointer)
  programChangeAttributes(stringPointer)

  programMakeFile(stringPointer)
  programMakeDirectory(stringPointer)

  programMove(stringPointer)
  programRemove(stringPointer)
  programEra(stringPointer)                             ' same as programremove

  programDump(stringPointer)

  programToggleLineFeed(stringPointer)

  ifnot(str.stringCompareCI(string("ver"), stringPointer))
    executeCommand(string("_VER.CMD"),stringPointer)
  ifnot(str.stringCompareCI(string("help"), stringPointer))
    executeCommand(string("_HELP.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("mount"), stringPointer))
    executeCommand(string("_MOUNT.CMD"),stringPointer)
  ifnot(str.stringCompareCI(string("unmount"), stringPointer))
    executeCommand(string("_UNMOUNT.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("free"), stringPointer))
    executeCommand(string("_FREE.CMD"),stringPointer)
  ifnot(str.stringCompareCI(string("used"), stringPointer))
    executeCommand(string("_USED.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("dir"), stringPointer))
    executeCommand(string("_DIR.CMD"),stringPointer)
  ifnot(str.stringCompareCI(string("ls"), stringPointer))
    executeCommand(string("_LS.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("cat"), stringPointer))
    executeCommand(string("_CAT.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("copy"), stringPointer))
    executeCommand(string("_COPY.CMD"),stringPointer)
  ifnot(str.stringCompareCI(string("diff"), stringPointer))
    executeCommand(string("_DIFF.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("test"), stringPointer))
    executeCommand(string("_TEST.CMD"),stringPointer)

  ifnot(str.stringCompareCI(string("format"), stringPointer))
    executeCommand(string("_FORMAT.CMD"),stringPointer)

'  programDate(stringPointer)                                      
'  programTime(stringPointer)

'  programRamSet(stringPointer)
'  programRamGet(stringPointer)

'  programSQW(stringPointer)
'  programOUT(stringPointer)

'  programPlayWavFile(stringPointer)
'  programTapeWavFile(stringPointer)

  programCommand(StringPointer) 'must be last! Is it a command ie an .exe file?

  abort string("Command Not Found! Try Help")
  crlf


PRI PrintDosVer
  PrintStringCR(string("*** KyeDOS SD card operating system v3.10 ***"))
  PrintString(string(  " by Kwabena W. Agyeman & J. Moxham & Cluso99 (hw="))
  case hardware >> 29
    1 : PrintString(string("RamBlade"))
    2 : PrintString(string("TriBlade#2"))
    3 : PrintString(string("RamBlade3"))
    other: PrintString(string("???"))
  PrintStringCR(string(")"))

PRI StopAll | t

  waitcnt(clkfreq * 5 + cnt)
  repeat t from 0 to 7                                  'Stop all COGs except this one
    if (t <> cogid)
      cogstop(t)
  cogstop(cogid)                                        'then stop this cog. All done!

PRI HardwareFail | pu
  PrintString(string("** Cannot determine hardware ** $"))
  sio.hex(0,pu,8)
  crlf
  StopAll

PRI shellLine(prompt)

  PrintString(prompt)
  repeat
    result := GetKey            ' scan all inputs (keyboard/comport)
    PrintChar(result)           ' send to vga and to com port
    str.buildString(result)     ' build an input string
  while((result <> _carriageReturnCharacter) and (result <> _lineFeedCharacter) and (result <> _nullCharacter))

  return str.trimString(str.builtstring(true))

PRI GetKey
  repeat
'   if key.gotkey <>0           ' scan both the local keyboard and the com port
'     result:=key.getkey
    if sio.rxavail(0)
      result :=sio.rx(0)  
  until result <>0              ' until something comes in either port
    
  if result== _backspaceKeycode ' backspace keycode is 200 but convert to 8
    result := _backspaceCharacter 

PRI shellDecision(prompt, characterToFind, otherCharacterToFind)

  PrintString(prompt~)  

  repeat
    result := GetKey            ' get a key from the input device (keyboard/serial)
    prompt or= ((result == characterToFind) or (result == otherCharacterToFind))
  while((result <> _carriageReturnCharacter) and (result <> _lineFeedCharacter) and (result <> _nullCharacter))

  return prompt  

PRI programClear(stringPointer)

  ifnot(str.stringCompareCI(string("clear"), stringPointer))
    pauseForMilliseconds(500)
    printstring(string(_clearScreenCharacter, _homeCursorCharacter))
    abort false

PRI programEcho(stringPointer)

  ifnot(str.stringCompareCI(string("echo"), stringPointer))
    printstring(str.trimString(stringPointer += 5))
    crlf
    abort false

PRI programReboot(stringPointer)

  ifnot(str.stringCompareCI(string("reboot"), stringPointer))
    printstring(string("Rebooting "))

    repeat 3
      pauseForMilliseconds(500)
      printstring(string(". "))

    pauseForMilliseconds(500)
    reboot

{{
DAT shellProgramHelpStrings

  byte "Command Listing..."
  byte 13
  byte "<clear>                        - Clear the screen."
  byte 13
  byte "<echo> <string>                - Echo the string argument."
  byte 13
  byte "<reboot>                       - Reboot the propeller chip."
  byte 13
  byte "<help>                         - Show the command listing."
  byte 13
  byte "<mount> <partition>            - Mount the file system."
  byte 13
  byte "<unmount>                      - Unmount the file system."
  byte 13
  byte "<free> <fast>                  - Free sector count. F=Fast. 1 Sector = 512 Bytes"
  byte 13
  byte "<used> <fast>                  - Used sector count. F=Fast. 1 Sector = 512 Bytes"
  byte 13
  byte "<ls> <all>                     - List contents of the working directory. A=All."
  byte 13
  byte "<dir>                          - List contents. Includes wildcards eg dir *.exe"
  byte 13
  byte "<cat> <file> <hex>             - Print the ASCII interpreted contents of a file."
  byte 13
  byte "<cd> <directory>               - Change directory."
  byte 13
  byte "<chmod> <entry> <attributes>   - Change Attrib: R=R/O, H=Hidden, S=Sys, A=Arch"
  byte 13
  byte "<mv> <oldPath> <newPath>       - Move a file or directory."
  byte 13
  byte "<rm> <path> or <era>           - Remove a file or directory."
  byte 13
  byte "<mkfile> <name>                - Make a new file."
  byte 13
  byte "<mkdir> <name>                 - Make a new directory."
  byte 13
  byte "<cp> <oldPath> <newPath>       - Copy a file."
  byte 13
  byte "<diff> <path> <path>           - Print differences between two files."
  byte 13
  byte "<boot> <file>                  - Reboot the propeller chip from a file."
  byte 13
  byte "<bin>                          - Display the <boot>able files."
  byte 13
  byte "<filename>                     - Reboot and run filename.exe"
  byte 13
  byte "<format> <partition>           - Format and mount the file system."
  byte 13
' byte "<date>                         - Display the current date and time."
' byte 13
' byte "<time>                         - Change the current date and time."
' byte 13
'  byte "<set> <byteNumber> <byteValue> - Set the value of a SRAM byte on the RTClock."
'  byte 13
'  byte "<get> <byteNumber>             - Get the value of a SRAM byte on the RTClock."
'  byte 13
'  byte "<sqw> <frequencyNumber>        - Set the frequecny of the SWQ / OUT pin. (0=1Hz, 1=4096Hz, 2=8192Hz, 3=32768Hz)."
'  byte 13
'  byte "<out> <digitalState>           - Set the state of the SWQ / OUT pin. (0=low, 1=high)."
'  byte 13
'  byte "<play> <file>                  - Play a WAV file."
'  byte 13
'  byte "<tape> <file>                  - Tape a WAV file."
'  byte 13
  byte "<dump>                         - Dump the ROM font."
  byte 13
  byte "<test>                         - Speed test the file system."
  byte 13
  byte "<togglelf>                     - Toggle LineFeed on/off."
  byte 13
  byte 0                                        'required at the end.
}}


PRI programChangeDirectory(stringPointer)

  ifnot(str.stringCompareCI(string("cd"), stringPointer))
    stringPointer := str.tokenizeString(0)
    fat.changeDirectory(stringPointer)
    taf.changeDirectory(stringPointer)
    abort 0

PRI programChangeAttributes(stringPointer)

  ifnot(str.stringCompareCI(string("chmod"), stringPointer))
    fat.changeAttributes(str.tokenizeString(0), str.tokenizeString(0))
    abort 0

PRI programMove(stringPointer)

  ifnot(str.stringCompareCI(string("mv"), stringPointer))
    fat.moveEntry(str.tokenizeString(0), str.tokenizeString(0))
    abort 0

PRI programRemove(stringPointer)

  ifnot(str.stringCompareCI(string("rm"), stringPointer)) 
    fat.deleteEntry(str.tokenizeString(0))
    abort 0

PRI programERA(stringPointer) ' same as ProgramRemove

  ifnot(str.stringCompareCI(string("era"), stringPointer)) 
    fat.deleteEntry(str.tokenizeString(0))
    abort 0

PRI programMakeFile(stringPointer)

  ifnot(str.stringCompareCI(string("mkfile"), stringPointer))
    fat.newFile(str.tokenizeString(0))
    abort 0

PRI programMakeDirectory(stringPointer)

  ifnot(str.stringCompareCI(string("mkdir"), stringPointer))
    fat.newDirectory(str.tokenizeString(0))
    abort 0


PRI programBoot(stringPointer)

  stringPointer := str.tokenizeString(stringPointer)
  ifnot(str.stringCompareCI(string("boot"), stringPointer))
    stringPointer := str.tokenizeString(0)
    'printstring(string("Are you sure you want to reboot from "))
    'printstring(stringPointer)
    'if(shellDecision(string(" - [y]es or [n]o? : "), "Y", "y"))
      fat.bootPartition(stringPointer)
    abort 0

PRI programBin(stringPointer)

  ifnot(str.stringCompareCI(string("bin"), stringPointer))
    DisplayBinaryFiles
    abort 0



PRI programDump(stringPointer) | outsideCounter, insideCounter

  ifnot(str.stringCompareCI(string("dump"), stringPointer))
    fat.openFile(fat.newFile(string("P8X32A.TXT")), "W")

    fat.writeString(string("P8X32A Character Set", _carriageReturnCharacter, _lineFeedCharacter, _carriageReturnCharacter, _lineFeedCharacter))
    repeat outsideCounter from 0 to 255

      printstring(string("Dumping character #"))
      printstringcr(1 + str.integerToDecimal(outsideCounter, 3))

      fat.writeString(string("Character #"))
      fat.writeString(1 + str.integerToDecimal(outsideCounter, 3))
      fat.writeString(string(_carriageReturnCharacter, _carriageReturnCharacter))
      fat.writeString(string(_carriageReturnCharacter, _carriageReturnCharacter))

      repeat result from 0 to 31
        repeat insideCounter from (outsideCounter & 1) to 31 step 2

          if(long[32_768][result + ((outsideCounter >> 1) << 5)] & (|<insideCounter))
            fat.writeByte(".")
          else
            fat.writeByte(" ")

        fat.writeString(string(_carriageReturnCharacter, _carriageReturnCharacter))
      fat.writeString(string(_carriageReturnCharacter, _carriageReturnCharacter))

    fat.closeFile
    abort 0


PRI ProgramToggleLineFeed(stringPointer)
  ifnot(str.stringCompareCI(string("togglelf"), stringPointer))
    if long[_hub#_toggleLF]                             'toggle LineFeed on/off
      long[_hub#_toggleLF] := false
    else
      long[_hub#_toggleLF] := true
    abort 0
    
PRI PauseForMilliseconds(ms)
    waitcnt(clkfreq/1000 * ms + cnt)


PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    sio.tx(0,c)                                         ' send to com port
{
    if (_VGA_Enabled)
      vt100.out(c)                                      ' send to vga
    if (_Serial_Enabled)
      sio.tx(0,c)                                       ' send to com port
    if (_lcd_Enabled)
      LCD.out(c)                                        ' send to LCD display        
}

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' send to vga
    if long[_hub#_toggleLF]
      PrintChar(_lineFeedCharacter)                     ' send to vga

PUB PrintDecimal(number)        ' print the decimal value, useful for debuggin
    PrintString(str.integerToDecimal(number, 10))    

PRI DisplayBinaryFiles | buffer
  PrintString(string("Use the BOOT command to run the following binary Spin programs/emulations:"))
  crlf
  fat.listEntries("W")   
  repeat while(buffer := fat.listEntries("N"))   
      buffer := str.trimstring(buffer)   
      if str.endswithstring(buffer,string(".BIN")) == -1
        PrintString(buffer)
        crlf
  crlf


PRI ProgramCommand(StringPointer) | buffer,i ' look for .exe files (ie .binary files renamed to .exe) and if found, run the program
    fat.listEntries("W")
    str.copy(stringpointer,@lineoftext2)                ' copy the command to lineoftext2
    str.stringConcatenate(@lineoftext2, string(".exe")) ' add extension for executable files to lineoftext2
     repeat while(buffer := fat.listEntries("N"))
       str.copy(buffer,@LineOfText1)                    ' copy filename to LineOfText1
       str.trimstring(@lineoftext1)                     ' remove any leading/trailing spaces  
       ifnot str.stringcompareci(@lineoftext1,@lineoftext2)' does it match? (ignore case)
         str.copy(buffer,@lineoftext3)                  ' can't use "buffer" as it gets overwritten
         'printstringcr(string("Saving command line in COMMAND.TXT and running program"))
         'printstring(string("Command line: "))
         'printstring(@commandline)
          ' above commented out as 1) about to blank the screen, 2) debugged and 3) if running xmodem don't want any further data on the serial lines
          ' possibly could put this output back if use the vga screen only.                   
         result := \fat.newFile(string("COMMAND.TXT")) ' All aborts traps must have "variable := \function" setup.
         if(fat.partitionError == fat#Entry_Already_Exist)  ' file already exists
            result := \fat.deleteEntry(string("COMMAND.TXT"))    ' delete the old file
            result := \fat.newfile(string("COMMAND.TXT")) ' recreate the file
         result := \fat.openFile(string("COMMAND.TXT"), "W") ' create file and open file
         result := \fat.writestring(@commandline)       ' store the command line as it was typed
         result := \fat.closefile                       ' close the file
         result := \fat.bootpartition(@lineoftext3)     ' reboot
    reboot                                              'failure if we get here!
    
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}