{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command module _xxxxxxx.CMD called by _CMD.CMD, returns to _CMD.CMD 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120306  XXX_014 make general purpose base module
//                
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON
  _clkmode = _hw#_clkmode
  _xinfreq = _hw#_xinfreq

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef"             'fixed hub definitions ($7FFF... down)

  fat   : "SD-MMC_FATEngine_001.spin"
  taf   : "SD-MMC_FATEngine_001.spin"
  str   : "ASCII0_STREngine.spin"
  sio   : "pcFullDuplexSerial2FC" 

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

   
PUB shell | z,z0,z1,z2,z3,z4

  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms

{{
'==================
'force parameters in hub from module _CMD.CMD
  long[_hub#_SDPins]  :=  23 << 24 | 22 << 16 | 21 << 8 | 20  '(RamBlade3)                   
  long[_hub#_SIOPins] := 0 << 16 | 30 << 8 | 31
  long[_hub#_SIOBaud] := 115200                    
  long[_hub#_toggleLF] := false
  str.copy(string("???"),_hub#_stringBuffer)                 
'==================
}}

'get the hub parameters...
  z  :=  long[_hub#_sioPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
' z3 := (z >> 24) & $1F
  z4 :=  long[_hub#_sioBaud]
  sio.AddPort(0, z0, z1, -1, -1, 0, z2, z4)
  sio.start                                             ' start the com port

  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card
'  crlf
  PrintStringCR(string("=== HELP ==="))                 ' program message

  ifnot(fat.partitionMounted)
    PrintString(string("==Card Not Mounted=="))

  sio.rxflush(0)                                      

'=================
' execute the command
' str.copy(string("???"),_hub#_stringBuffer)                 
  str.copy(_hub#_stringBuffer,@commandline)             ' retrieve commandline 
  str.copy(@commandline,result)
  result := str.tokenizeString(result)
  result := \executeCommand(result)
'=================

' return to the command module
  \fat.bootPartition(string("_CMD.CMD"))
  'only here on error
  PrintStringCR(string("=== error ==="))
  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
  reboot

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

PUB PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PUB PrintDecimal(number)        ' print the decimal value, useful for debugging
    PrintString(str.integerToDecimal(number, 10))    

PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    sio.tx(0,c)                                         ' send to com port

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output devices
    if long[_hub#_toggleLF]
      PrintChar(_lineFeedCharacter)                     ' sends <lf> to output devices

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}
 
PRI executeCommand(stringPointer)

  ifnot(str.stringCompareCI(string("help"), stringPointer))
    printstring(@HelpStrings)
    abort 0

DAT HelpStrings

  byte 13
  byte "Command Listing"
  byte 13
  byte "<clear>                        - Clear the screen."
  byte 13
  byte "<echo> <string>                - Echo the string argument."
  byte 13
  byte "<reboot>                       - Reboot the propeller chip."
  byte 13
  byte "<help>                         - Show the command listing."
  byte 13
  byte "<mount> <partition>            - Mount the file system."
  byte 13
  byte "<unmount>                      - Unmount the file system."
  byte 13
  byte "<free>                         - Free sector count. 1 Sector = 512 Bytes"
  byte 13
  byte "<used>                         - Used sector count. 1 Sector = 512 Bytes"
  byte 13
  byte "<ls> <all>                     - List contents of the working directory. A=All."
  byte 13
  byte "<dir>                          - List contents. Includes wildcards eg dir *.exe"
  byte 13
  byte "<cat> <file> <hex>             - Print the ASCII interpreted contents of a file."
  byte 13
  byte "<cd> <directory>               - Change directory."
  byte 13
  byte "<chmod> <entry> <attributes>   - Change Attrib: R=R/O, H=Hidden, S=Sys, A=Arch"
  byte 13
  byte "<mv> <oldPath> <newPath>       - Move a file or directory."
  byte 13
  byte "<rm> <path> or <era>           - Remove a file or directory."
  byte 13
  byte "<mkfile> <name>                - Make a new file."
  byte 13
  byte "<mkdir> <name>                 - Make a new directory."
  byte 13
  byte "<copy> <oldPath> <newPath>     - Copy a file."
  byte 13
  byte "<diff> <path> <path>           - Print differences between two files."
  byte 13
  byte "<boot> <file>                  - Reboot the propeller chip from a file."
  byte 13
  byte "<filename>                     - Reboot and run filename.exe"
  byte 13
  byte "<format> <partition>           - Format and mount the file system."
  byte 13
' byte "<date>                         - Display the current date and time."
' byte 13
' byte "<time>                         - Change the current date and time."
' byte 13
'  byte "<set> <byteNumber> <byteValue> - Set the value of a SRAM byte on the RTClock."
'  byte 13
'  byte "<get> <byteNumber>             - Get the value of a SRAM byte on the RTClock."
'  byte 13
'  byte "<sqw> <frequencyNumber>        - Set the frequecny of the SWQ / OUT pin. (0=1Hz, 1=4096Hz, 2=8192Hz, 3=32768Hz)."
'  byte 13
'  byte "<out> <digitalState>           - Set the state of the SWQ / OUT pin. (0=low, 1=high)."
'  byte 13
'  byte "<play> <file>                  - Play a WAV file."
'  byte 13
'  byte "<tape> <file>                  - Tape a WAV file."
'  byte 13
  byte "<dump>                         - Dump the ROM font."
  byte 13
  byte "<test>                         - Speed test the file system."
  byte 13
  byte 0                ' required last         

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}