{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command module _xxxxxxx.CMD called by _CMD.CMD, returns to _CMD.CMD 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120306  XXX_014 make general purpose base module
//                
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON
  _clkmode = _hw#_clkmode
  _xinfreq = _hw#_xinfreq

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef"             'fixed hub definitions ($7FFF... down)

  fat   : "SD-MMC_FATEngine_001.spin"
  taf   : "SD-MMC_FATEngine_001.spin"
  str   : "ASCII0_STREngine.spin"
  sio   : "pcFullDuplexSerial2FC" 

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

   
PUB shell | z,z0,z1,z2,z3,z4

  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms

{{
'==================
'force parameters in hub from module _CMD.CMD
  long[_hub#_SDPins]  :=  23 << 24 | 22 << 16 | 21 << 8 | 20  '(RamBlade3)                   
  long[_hub#_SIOPins] := 0 << 16 | 30 << 8 | 31
  long[_hub#_SIOBaud] := 115200                    
  long[_hub#_toggleLF] := false
  str.copy(string("???"),_hub#_stringBuffer)                 
'==================
}}

'get the hub parameters...
  z  :=  long[_hub#_sioPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
' z3 := (z >> 24) & $1F
  z4 :=  long[_hub#_sioBaud]
  sio.AddPort(0, z0, z1, -1, -1, 0, z2, z4)
  sio.start                                             ' start the com port

  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card
'  crlf
  PrintStringCR(string("=== TEST ==="))                 ' program message

  ifnot(fat.partitionMounted)
    PrintString(string("==Card Not Mounted=="))

  sio.rxflush(0)                                      

'=================
' execute the command
' str.copy(string("???"),_hub#_stringBuffer)                 
  str.copy(_hub#_stringBuffer,@commandline)             ' retrieve commandline 
  str.copy(@commandline,result)
  result := str.tokenizeString(result)
  result := \executeCommand(result)
'=================

' return to the command module
  \fat.bootPartition(string("_CMD.CMD"))
  'only here on error
  PrintStringCR(string("=== error ==="))
  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
  reboot

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

PUB PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PUB PrintDecimal(number)        ' print the decimal value, useful for debugging
    PrintString(str.integerToDecimal(number, 10))    

PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    sio.tx(0,c)                                         ' send to com port

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output devices
    if long[_hub#_toggleLF]
      PrintChar(_lineFeedCharacter)                     ' sends <lf> to output devices

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}
 
PRI executeCommand(stringPointer) | counter, buffer

  ifnot(str.stringCompareCI(string("test"), stringPointer))
    fat.newFile(stringPointer := string("P8X32A.ROM"))

    buffer := startTest(stringPointer, "W", string("Byte write test...   "))
    repeat counter from 32_768 to 65_535 step 1
      fat.writeByte(byte[counter])
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Byte read test...    "))
    repeat counter from 32_768 to 65_535 step 1
      if(fat.readByte <> byte[counter])
        fat.closeFile
        abort string("Byte read back failed")
        crlf
    stopTest("R", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "W", string("Word write test...   "))
    repeat counter from 32_768 to 65_535 step 2
      fat.writeShort(word[counter])
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Word read test...    "))
    repeat counter from 32_768 to 65_535 step 2
      if(fat.readShort <> word[counter])
        fat.closeFile
        abort string("Word read back failed")
        crlf
    stopTest("R", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "W", string("Long write test...   "))
    repeat counter from 32_768 to 65_535 step 4
      fat.writeLong(long[counter])
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Long read test...    "))
    repeat counter from 32_768 to 65_535 step 4
      if(fat.readLong <> long[counter])
        fat.closeFile
        abort string("Long read back failed")
        crlf
    stopTest("R", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "W", string("Sector write test... "))
    repeat counter from 32_768 to 65_535 step 512
      fat.writeData(counter, 512)
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Sector read test...  "))
    repeat counter from 32_768 to 65_535 step 512
      fat.readData(counter, 512)
    stopTest("R", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "W", string("Memory write test... "))
    repeat counter from 32_768 to 65_535 step 32_768
       fat.writeData(counter, 32_768)
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Memory read test...  "))
    repeat counter from 32_768 to 65_535 step 32_768
      fat.readData(counter, 32_768)
    stopTest("R", 32_768, (cnt - buffer))

    fat.newFile(fat.deleteEntry(stringPointer))
    buffer := startTest(stringPointer, "A", string("Block append test... "))
    fat.writeData(32_768, 32_768)
    stopTest("W", 32_768, (cnt - buffer))

    buffer := startTest(stringPointer, "R", string("Reverse seek test... "))
    repeat counter from 32_767 to 0 step 512
      fat.fileSeek(counter)
    stopTest("R", 32_768, (cnt - buffer))

    fat.deleteEntry(stringPointer)

    abort 0

PRI startTest(path, mode, message)

  fat.openFile(path, mode)
  printstring(message)
  return cnt

PRI stopTest(mode, count, cycles)

  if(mode == "W")
    printstring(string("Write "))
  else
    printstring(string("Read  "))

  printstring(1 + str.integerToDecimal((count >> 10), 3))
  printstring(string(" KBs at "))
  printstring(1 + str.integerToDecimal((multiplyDivide(count, cycles) >> 10), 3))
  printstringcr(string(" KBs per second"))
  fat.closeFile

PRI multiplyDivide(dividen, divisor) | productHigh, productLow

  productHigh := clkfreq ** dividen
  productLow := clkfreq * dividen
  if(productHigh => divisor)
    return posx

  repeat 32

    dividen := (productHigh < 0)
    productHigh := ((productHigh << 1) + (productLow >> 31))
    productLow <<= 1
    result <<= 1

    if((productHigh => divisor) or dividen)
      productHigh -= divisor
      result += 1

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}